package de.intevation.flys.artifacts;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import de.intevation.artifactdatabase.data.DefaultStateData;

import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.State;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactFactory;
import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.flys.artifacts.model.WKms;
import de.intevation.flys.artifacts.model.WKmsFacet;
import de.intevation.flys.artifacts.model.WstValueTable;
import de.intevation.flys.artifacts.model.WstValueTableFactory;
import de.intevation.flys.artifacts.model.WKmsFactory;

import de.intevation.flys.artifacts.states.StaticState;
import de.intevation.flys.artifacts.resources.Resources;

import de.intevation.flys.utils.FLYSUtils;
import de.intevation.artifacts.common.utils.XMLUtils;


/**
 * Artifact to access additional "waterlevel"-type of data, like the height
 * of protective measures (dikes).
 *
 * This artifact neglects (Static)FLYSArtifacts capabilities of interaction
 * with the StateEngine by overriding the getState*-methods.
 */
public class StaticWKmsArtifact
extends      StaticFLYSArtifact
{
    /** The logger for this class. */
    private static Logger logger =
        Logger.getLogger(StaticWKmsArtifact.class);

    /** XPath to access initial parameter. */
    public static final String XPATH_DATA =
        "/art:action/art:ids/@value";

    /** One and only state to be in. */
    protected transient State state = null;


    /**
     * Trivial Constructor.
     */
    public StaticWKmsArtifact() {
        logger.debug("StaticWKmsArtifact.StaticWKmsArtifact");
    }


    /**
     * Gets called from factory, to set things up.
     */
    @Override
    public void setup(
        String          identifier,
        ArtifactFactory factory,
        Object          context,
        CallMeta        callMeta,
        Document        data)
    {
        logger.debug("StaticWKmsArtifact.setup");

        state = new StaticState(
            "state.additional_wkms.static",
            "state.additional_wkms.static");

        List<Facet> fs = new ArrayList<Facet>();
        logger.debug(XMLUtils.toString(data));
        String code = XMLUtils.xpathString(
            data, XPATH_DATA, ArtifactNamespaceContext.INSTANCE);

        logger.debug("makes: " + code);

        // TODO Go for JSON, one day.
        //ex.: flood_protection-wstv-114-12
        if (code != null) {
            String [] parts = code.split("-");

            if (parts.length >= 4) {
                try {
                    Integer.valueOf(parts[2]);
                    Integer.valueOf(parts[3]);
                    addStringData("col_pos", parts[2]);
                    addStringData("wst_id",  parts[3]);
                }
                catch (Exception e) {}
            }
        }

        Facet facet = new WKmsFacet(Resources.getMsg(callMeta,
                   "facet.discharge_curves.mainvalues.w",
                   "facet.discharge_curves.mainvalues.w"));
        fs.add(facet);
        facets.put(state.getID(), fs);

        spawnState();
        super.setup(identifier, factory, context, callMeta, data);
    }


    /**
     * Initialize the static state with output.
     * @return static state
     */
    protected State spawnState() {
        state = new StaticState(
            "state.additional_wkms.static",
            "state.additional_wkms.static");
        List<Facet> fs = (List<Facet>) facets.get("state.additional_wkms.static");
        DefaultOutput output = new DefaultOutput(
            "w_differences",
            "w_differences.longitudinal_section", "image/png",
            fs,
            "chart");
        // TODO: also add longitudinal_section output?

        state.getOutputs().add(output);
        return state;
    }


    /**
     * Called via setup.
     */
    @Override
    protected void initialize(Artifact artifact, Object context, CallMeta meta) {
        logger.debug("StaticWKmsArtifact.initialize");
        WINFOArtifact winfo = (WINFOArtifact) artifact;
        // TODO: The river is of no interest, so far.
        addData("river", winfo.getData("river"));
    }


    /**
     * Get a list containing the one and only State.
     * @param  context ignored.
     * @return list with one and only state.
     */
    @Override
    protected List<State> getStates(Object context) {
        ArrayList<State> states = new ArrayList<State>();
        states.add(getState());
        return states;
    }


    /**
     * Get the "current" state (there is but one).
     * @param cc ignored.
     * @return the "current" (only possible) state.
     */
    @Override
    public State getCurrentState(Object cc) {
        return getState();
    }


    /**
     * Get the only possible state.
     * @return the state.
     */
    protected State getState() {
        return getState(null, null);
    }


    /**
     * Get the state.
     * @param context ignored.
     * @param stateID ignored.
     * @return the state.
     */
    @Override
    protected State getState(Object context, String stateID) {
        if (state != null)
            return state;
        else
            return spawnState();
    }


    /**
     * Get WKms from factory.
     * @param TODO idx param is not needed
     * @return WKms according to parameterization (can be null);
     */
    public WKms getWKms(int idx) {
        logger.debug("StaticWKmsArtifact.getWKms");

        return WKmsFactory.getWKms(
            Integer.valueOf(getDataAsString("col_pos")),
            Integer.valueOf(getDataAsString("wst_id")));
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
